class ApplicationController < ActionController::Base
  include TabNav::ControllerMethods

  protect_from_forgery with: :exception

  respond_to :html

  before_filter :do_not_cache
  unless App.test?
    before_filter :log_level
  end
  before_filter :load_settings
  before_filter :validate_host
  before_filter :set_ip_address
  before_filter :authenticate
  before_filter :deleted_account
  before_filter :set_current_user  # keep this early so events can make use of it
  before_filter :set_time_zone
  before_filter :expired_account
  before_filter :disabled_account
  before_filter :check_session_timeout
  before_filter :reset_session_filter
  before_filter :check_password_reset
  before_filter :logout_on_invalid_license
  before_filter :redirect_to_license_if_invalid
  before_filter :set_protocol_host_port
  before_filter :set_error_count
  before_filter :update_last_request_at, :if => :logged_in?

  clear_helpers
  helper :link, :table

  helper_method :current_user
  helper_method :ui_manage_permissions?
  helper_method :api_manage_permissions?
  helper_method :logged_in?
  helper_method :ami?
  helper_method :saml_enabled?
  helper_method :available_home_shares
  helper_method :available_shares

  API_LOGIN = 'api.login'
  APP_LOGIN = 'app.login'
  APP_ADMIN = 'app.admin'

  BUILD_IN_PERMISSIONS = [APP_ADMIN, APP_LOGIN, API_LOGIN]

  # or maybe this should look like a 500?
  NOT_AUTHORIZED_MESSAGE = "You are not authorized to view the requested page."
  ACCOUNT_DISABLED_MESSAGE = 'Your account has been disabled. Contact your admin.'
  ACCOUNT_EXPIRED_MESSAGE = 'Your account has expired. Contact your admin.'
  ACCOUNT_NOT_FOUND_MESSAGE = 'Account not found.  Contact your admin.'

  begin 'exception handling'

    if App.handle_uncaught_errors?

      rescue_from Exception do |e|
        @error = Aspera::Error.handle(e, :log_to_database => true)
        if request.xhr?
          # redirect to error page using window.location (Firefox < version 7 doesn't do redirects
          # in ajax requests properly) - https://bugzilla.mozilla.org/show_bug.cgi?id=553888
          render 'shared/redirect_to_error', :status => :internal_server_error
        else
          redirect_to error_500_path(:id => @error.database_id)
        end
      end

      rescue_from ActiveRecord::RecordNotFound do
        redirect_to not_found_path
      end

      rescue_from ActionView::MissingTemplate do
        head :not_acceptable
      end
    end

  end

  protected

  def accepted_hosts
    ip = Socket.ip_address_list.detect { |local_ip| local_ip.ipv4_private? }
    @accepted_hosts ||= (SHARES_CONFIG['AcceptedHosts'] || [])
    if ip != nil
      ip = ip.ip_address
      hostname = Socket.gethostname
      @accepted_hosts << ip if !@accepted_hosts.include?(ip)
      @accepted_hosts << hostname if !@accepted_hosts.include?(hostname)
    end
    return @accepted_hosts
  end

  def validate_host
    unless accepted_hosts.include?(request.host)
      render text: 'Invalid hostname', status: 404
    end
  end

  def check_flag(flag_name)
    unless Aspera::Flag.on?(flag_name)
      flash[:error] = 'Feature not enabled'
      redirect_to root_url
      false
    end
  end

  # Only an admin could reach this code, as a user would previously be auto-logged out
  # and not be able to login when license is invalid
  def redirect_to_license_if_invalid
    if !valid_license? && !request.xhr?
      flash[:error] = ('Missing or invalid license, or license terms exceeded')
      if Aspera::Config::EntitlementLicenseMode.enabled?
        redirect_to config_entitlement_license_path
      else
        redirect_to config_license_path
      end
    end
  end

  def set_ip_address
    Authorizable::User.ip_address = request.remote_ip
  end

  def set_current_user
    Authorizable::User.current = current_user.try(:authorizable_user)
  end

  def set_protocol_host_port
    Authentication::Server::Service.protocol_host_port = protocol_host_port
  end

  def authenticate
    redirect_to_login unless logged_in?
  end

  # Logout all non-admin users if license invalid
  def logout_on_invalid_license
    if !valid_license? && !current_user.admin?
      reset_session
      session[:return_to] = request.fullpath
      redirect_to_login(:local => true)
    end
  end

  def deleted_account
    if current_user && current_user.authorizable_user.nil?
      session.delete(:user)
      reset_session
      flash[:error] = ACCOUNT_NOT_FOUND_MESSAGE
      redirect_to_login(:local => true)
    end
  end

  def expired_account
    if logged_in? && current_user.account_expired?
      reset_session
      flash[:warning] = ACCOUNT_EXPIRED_MESSAGE
      redirect_to_login(:local => true)
    end
  end

  def disabled_account
    if logged_in? && current_user.disabled?
      reset_session
      flash[:warning] = ACCOUNT_DISABLED_MESSAGE
      redirect_to_login(local: true)
    end
  end

  def check_session_timeout
    if session[:expires_at].blank? || session[:expires_at] < Time.now
      SessionTimedOutEvent.create_event
      reset_session
      flash[:warning] = 'Your session has timed out. Log in again to continue.'
      redirect_to_login(local: true)
    end
  end

  if Rails.env.development?
    def check_session_timeout
    end
  end

  def reset_session_filter
    if !(session[:expires_at].blank? || session[:expires_at] < Time.now)
      reset_session_expiration
    end
  end

  def reset_session_expiration(time_out_at = nil)
    default_timeout_at = Aspera::Config::UserSecurity.load_settings.session_timeout.minutes.from_now
    # TODO: this needs to deal w/TZ
    expires_at = [default_timeout_at, time_out_at].compact.min
    session[:expires_at] = expires_at
  end

  def redirect_to_login(options = {})
    if request.xml_http_request?
      session[:return_to] = request.referer
      render :js => %|document.location.href = "#{login_path(local: params[:local] || options[:local])}";|
    else
      if request.get?
        session[:return_to] = request.fullpath
      else
        session[:return_to] = root_path #request.referer
      end
      redirect_to login_path(local: params[:local] || options[:local])
    end
  end

  def check_password_reset
    if logged_in? && current_user.password_reset_required?
      reset_session_expiration(1.minute.from_now)
      flash[:warning] = current_user.password_reset_warning
      redirect_to preferences_password_path
    end
  end

  def admin
    unless current_user.admin?
      return not_authorized!
    end
  end

  def admin_or_manager
    if !current_user.admin? && !current_user.manager?
      return not_authorized!
    end
  end

  def not_authorized!
    flash[:error] = NOT_AUTHORIZED_MESSAGE
    redirect_to root_url
    false
  end

  def current_user
    @current_user ||= User.new(session[:user]) if session[:user]
  end

  def valid_license?
    @valid_license ||= AsperaSharesLicense.new.valid?
  end

  def available_home_shares
    @available_home_shares ||= current_user.available_home_shares
  end

  def available_shares
    @available_shares ||= current_user.available_shares
  end

  def set_authorizable_user
    @authorizable_user = current_user.authorizable_user
  end

  def logged_in?
    current_user.present?
  end

  def sanitize(str)
    ActionController::Base.helpers.sanitize(str)
  end

  def ami?
    unless App.ami?
      redirect_to redirect_to_login
    end
  end

  def set_time_zone
    if logged_in?
      Time.zone = current_user.effective_time_zone
    end
  end

  def set_error_count
    if logged_in? && current_user.try(:authorizable_user).try(:admin?)
      @error_count = Aspera::Correctness::Item.errors.count
    end
  end

  def update_last_request_at
    current_user.try(:authorizable_user).try(:touch, :last_request_at)
  end

  def protocol_host_port
    "#{request.protocol}#{request.host_with_port}"
  end

  def do_not_cache
    response.headers["Cache-Control"] = "no-cache, no-store, max-age=0, must-revalidate"
    response.headers["Pragma"] = "no-cache"
    response.headers["Expires"] = "Fri, 01 Jan 1990 00:00:00 GMT"
  end

  # Guard against deleting a non-existent record:
  #
  #   rescued_destroy { User.find(params[:id]) }
  #   rescued_destroy(:redirect_to => root_url) { User.find(params[:id]) }
  #   rescued_destroy(:notice => "User was deleted.") { User.find(params[:id]) }
  #   rescued_destroy(:log_error => true) { User.find(params[:id]) }
  #
  def rescued_destroy(options = {})
    options.reverse_merge!(:redirect_to => :back, :notice => "Deleted.")
    model = yield
    model.destroy
    redirect_to options[:redirect_to], :notice => options[:notice]
  rescue ActiveRecord::RecordNotFound => e
    Aspera::Error.handle(e) if options[:log_error]
    redirect_to options[:redirect_to], :notice => options[:notice]
  end

  # Renders approximately $(this).html(new_content):
  # The "new_content" defaults to a partial with the same name as the action.
  #
  #   render_js_replace  #=> $(this).html("<%= escape_javascript(render(params[:action])) %>");
  #
  def render_js_replace(partial=nil)
    partial ||= params[:action]
    render :file => 'shared/replace', :formats => [:js], :locals => {:partial => partial, :dom_id => params[:dom_id]}
  end

  def load_settings
    App.protocol_host_port = nil
    # TODO: load config settings in a before filter once.
  end

  def log_level
    Rails.logger.level = Aspera::Config::Logging.web_application
  end

  def saml_enabled?
    saml_config = SamlConfig.first
    saml_config && saml_config.enabled?
  end

  def redirect_to_back(default = root_path)
    redirect_to :back
  rescue ActionController::RedirectBackError
    redirect_to default
  end

  def ui_manage_permissions?
    Aspera::Config::ManagerPermission.allow_ui_management
  end

  def api_manage_permissions?
    Aspera::Config::ManagerPermission.allow_api_management
  end
end
