class SessionsController < LoginBaseController

  LOGIN_SUCCESS_MESSAGE = "Logged in"
  LOGGED_OUT_MESSAGE = "Logged out"
  TIMEOUT_MESSAGE = "Internal error during login"

  skip_before_filter(
    :authenticate,
    :deleted_account,
    :expired_account,
    :disabled_account,
    :redirect_to_license_if_invalid,
    :logout_on_invalid_license,
    :check_password_reset,
    :check_session_timeout,
    :locked_account,
    :set_time_zone,
    )

  before_filter :set_login_page_message

  def new
    if !session[:login_uuid].nil?
      reset_session
      if saml_enabled?
        redirect_to login_path(local: 'true')
      else
        redirect_to login_path
      end
      return
    end
    session[:login_uuid] = SecureRandom.uuid

    if logged_in?
      redirect_to root_path
      return
    end

    if saml_enabled? && !(params[:local] == 'true') && SamlConfig.first.enable_redirect?
      redirect_to saml_login_url
      return
    end

    if Aspera::Config::LoginScreen.allow_directory_selection
      @directories = AuthorizableDirectory.directory_servers
    end

    # Updates entitlement license before login in order to automatically decide if entitlement box is needed
    if Aspera::Config::EntitlementLicenseMode.enabled? && !Aspera::Config::Entitlement.safe_net_shares_enabled?
      Aspera::Config::Entitlement.load_settings.refresh_safe_net_data
    end

    @license_invalid = !valid_license?
  end

  def create
    if is_saml_response?
      flash.now[:error] = "SAML Identity Providers should post to #{saml_callback_url}"
      render :new
      return
    end
    params[:username] = sanitize(params[:username])
    params[:password] = params[:password]

    login
  end

  def new_token
  end

  def create_token
    if !EmailValidator.valid?(params[:email])
      flash.now[:error] = 'Email is specified with invalid format'
      render :new_token and return
    end
    authorizable_user ||= ForgotPasswordEvent.find_user(params[:email])
    authorizable_user.directory_user.request_password_reset! if authorizable_user && authorizable_user.local?
    # TODO: force anybody logged in as with that email to log out.

    ForgotPasswordEvent.create_event(params[:email])
  end

  def create_with_token
    auth_response = UserWithToken.authenticate(params[:token])
    if auth_response.success?
      login_success(auth_response.user_resource)
    else
      flash[:error] = auth_response.message
      redirect_to new_login_token_path
    end
  rescue => e
    default_message = "Error authenticating user with token"
    message = Aspera::Error.message(e, :default => default_message, :log_to_database => true)
    flash.now[:error] = message
    render "new"
  end

  def test_create
    auth_response = User.authenticate('', params[:username], params[:password], APP_LOGIN)

    if auth_response.error?
      # This action is a convenience for testing.
      # Tests should never submit bad credentials.
      render inline: '<p>TEST LOGIN FAILED</p>', status: :bad_request
    else
      session[:user] = auth_response.user_resource
      session[:expires_at] = 10.minutes.from_now

      render inline: '<p>Test Login Successful</p>'
    end
  end if Rails.env.test?

  def destroy
    current_user.try(:authorizable_user).try(:process_logout)
    logout = logout_url
    reset_session
    redirect_to logout, :notice => LOGGED_OUT_MESSAGE
  end

  def expire
    session[:expires_at] = Time.now
    render :js => "alert('Done.');"
  end if Rails.env.development?

  def health_check
    response = {}

    healthy = valid_license?
    license_type = "valid_license"
    begin
      license_type = Aspera::Config::Entitlement.entitlement_licensing_mode_valid? ? :entitled : :valid_license
    rescue Exception => e
      raise e if e.message !~ /Entitlement Licensing is only available on Linux/
    end
    response[license_type] = healthy

    # Node status
    response[:nodes] = []
    Node::Node.all.each do |node|
      fasp_node_status = StatsCollector::Node.cluster_status_message_for(node.host)
      fasp_node_status = "Active" if fasp_node_status == nil
      response[:nodes] << {
          id: node.id,
          status: fasp_node_status                                                                                                                            
      }

      if fasp_node_status != "Active"
        healthy = false
      else
        
      end
    end

    render json: response, status: (healthy ? 200 : 500)
  end

  def logged_out; end

  private

  def set_login_page_message
    @login_page_message = Aspera::Config::Message.load_settings.login_page_message
  end

  def authenticate_with_params
    directory_id = if Aspera::Config::LoginScreen.allow_directory_selection
      params[:directory_id]
    else
      # authenticate serially
      ''
    end

    Authentication::Server::Service::User.authenticate(
      directory_id: directory_id,
      username: params[:username],
      password: params[:password],
      required_permission: APP_LOGIN
    )
  end

  def is_saml_response?
    params[:SAMLResponse].present? && params[:username].blank? && params[:password].blank?
  end

  def logout_url
    if saml_enabled? && current_user.try(:authorizable_user).try(:saml?)
      logged_out_url
    else
      login_url(local: true)
    end
  end

end
